﻿using System;
using System.Collections.Generic;
using UnityEngine;


/// <summary>
/// Generates hexagonal shaped grid of hexagons.
/// </summary>
[ExecuteInEditMode()]
class HexagonalHexGridGenerator : ICellGridGenerator
{

    public GameObject HexagonPrefab;
    public int Radius;
    public List<Cell> hexagons = new List<Cell>();

    [Range(15.0f,20.5f)]
    public float fifthlayer;

    [Range(14.0f, 20.5f)]
    public float fourthlayer;

    [Range(14.0f, 20.5f)]
    public float thirdlayer;
    [Range(10.0f, 20.5f)]
    public float secondlayer;
    [Range(9.0f, 20.5f)]
    public float firstlayer;
    public override List<Cell> GenerateGrid()
    {
        hexagons.Clear();
        float cx=  0.0f;
        float cz = 19.1f;
        float cy = 0.50f;
        if (HexagonPrefab.GetComponent<Hexagon>() == null)
        {
            Debug.LogError("Invalid hexagon prefab provided");
            return hexagons;
        }
        int nr = 1;

        for (int i = 0; i < Radius; i++)
        {
            for (int j = 0; j < (Radius*2) - i - 1; j++)
            {
                GameObject hexagon = Instantiate(HexagonPrefab);

                hexagon.GetComponent<Cell>().pos = new Vector3(hexagon.transform.position.x, hexagon.transform.position.y, hexagon.transform.position.z);

                var hexSize = hexagon.GetComponent<Cell>().GetCellDimensions();
                
                if(nr == 11) {
                    hexagon.GetComponent<Hexagon>().type = "center";

                        
                }
            //    hexagon.transform.Rotate(new Vector3(-90, 0, 0));
                hexagon.transform.position = new Vector3((i * hexSize.x * 0.75f),0,(i * hexSize.z * 0.5f) + (j * hexSize.z));
                hexagon.GetComponent<Hexagon>().OffsetCoord = new Vector3(i,0, Radius - j - 1 - (i/2));
                hexagon.GetComponent<Hexagon>().HexGridType = HexGridType.odd_q;
                hexagon.GetComponent<Hexagon>().MovementCost = 1;
                hexagons.Add(hexagon.GetComponent<Cell>());
                
                hexagon.transform.parent = CellsParent;
                hexagon.GetComponent<Hexagon>().nr = nr++;
                
                if (i == 0) continue;

                GameObject hexagon2 = Instantiate(HexagonPrefab);
                hexSize = hexagon2.GetComponent<Cell>().GetCellDimensions();
                hexagon2.transform.position = new Vector3((-i * hexSize.x * 0.75f),0, (i * hexSize.z * 0.5f) + (j * hexSize.z));


                hexagon2.GetComponent<Cell>().pos = new Vector3(hexagon2.transform.position.x,0, hexagon2.transform.position.z);

                hexagon2.GetComponent<Hexagon>().OffsetCoord = new Vector3(-i,0, Radius - j - 1 - (i/2));
                hexagon2.GetComponent<Hexagon>().HexGridType = HexGridType.odd_q;
                
                hexagon2.GetComponent<Hexagon>().MovementCost = 1;
                hexagons.Add(hexagon2.GetComponent<Cell>());

                hexagon2.transform.parent = CellsParent;

                hexagon2.GetComponent<Hexagon>().nr = nr++;
            }
        }

        foreach(Cell cell in hexagons)
        {
            float dist = Vector2.Distance(new Vector2(cx, cz), new Vector2(cell.transform.position.x, cell.transform.position.z));


            if (dist > 16.53f && dist < 20.0f)
            {
                cell.GetComponentInParent<Hexagon>().type = "5th";
            }
            if (dist > 15.0f && dist < 16.5f)
            {   
                cell.GetComponentInParent<Hexagon>().type = "4th";
            }
            if (dist > 13.39f && dist < 15.1f)
            {
                cell.GetComponentInParent<Hexagon>().type = "3rd";

            }
            if (dist > 11.55f && dist < 13.39f)
            {
                cell.GetComponentInParent<Hexagon>().type = "2nd";
            }
            if (dist > 9.7f && dist < 11.55f)
            {
                cell.GetComponentInParent<Hexagon>().type = "1st";
            }
            if (dist < 9.7f)
            {
                cell.GetComponentInParent<Hexagon>().type = "begin";
            }







        }
        return hexagons;
    }







}

